#ifndef __VNC_VIEWER__
#define __VNC_VIEWER__

#include "rfb.h"
#include <gtk/gtkwidget.h>
#include <gtk/gtkadjustment.h>

#define VNC_VIEWER(obj)      GTK_CHECK_CAST(obj, vnc_viewer_get_type(), VncViewer)
#define VNC_VIERWER_CLASS(k) GTK_CHECK_CLASS_CAST(obj, vnc_viewer_get_type(), VncViewerClass)
#define VNC_IS_VIEWER(obj)   GTK_CHECK_TYPE(obj, vnc_viewer_get_type())

typedef struct _VncViewer VncViewer;
typedef struct _VncViewerClass VncViewerClass;

typedef enum {
  VNC_VIEWER_DEPTH_DEFAULT = -1,
  VNC_VIEWER_DEPTH_8BIT    = 8,
  VNC_VIEWER_DEPTH_16BIT   = 16,
  VNC_VIEWER_DEPTH_24BIT   = 24
} VncViewerDepthType;

typedef enum {
  VNC_VIEWER_CURSOR_NORMAL,
  VNC_VIEWER_CURSOR_DOT,
  VNC_VIEWER_CURSOR_NONE
} VncViewerCursorType;

struct _VncViewer {
  GtkWidget parent;

  VncViewerCursorType cursor_type;

  gint fd; /* the file descriptor for the RFB connection */
  
  GtkAdjustment *hadj, *vadj;

  /* these tags are from gdk_input_add when we are waiting on the fd */
  guint in_tag, out_tag;

  /* list of queued server bound messages */
  GList *out_queue;

  guint16 width, height;
  rfbPixelFormat pixel_format;
  gchar *desk_name;

  guint8 button_mask;

  GdkPixmap *offscreen;
  GdkGC *gc;

  GdkCursor *none, *dot;
};

struct _VncViewerClass {
  GtkWidgetClass parent_class;

  /* so it can be packed into a scrolled window ... */
  void (* set_scroll_adjustments)     (VncViewer *vnc, GtkAdjustment *hadj,
				       GtkAdjustment *vadj);

  /* handle the various server messages */
  void (* vnc_framebuffer_update)     (VncViewer *vnc,
				       rfbServerToClientMsg *msg);
  void (* vnc_set_colourmap_entries)  (VncViewer *vnc,
				       rfbServerToClientMsg *msg,
				       guint16 *entries);
  void (* vnc_bell)                   (VncViewer *vnc,
				       rfbServerToClientMsg *msg);
  void (* vnc_server_cut_text)        (VncViewer *vnc,
				       rfbServerToClientMsg *msg,
				       gchar *text);
  void (* connection_died)            (VncViewer *vnc);
};

GtkType vnc_viewer_get_type(void);

GtkWidget *vnc_viewer_new(gint fd, gboolean shared, VncViewerDepthType depth);
void vnc_viewer_configure(VncViewer *vnc, gint fd, gboolean shared,
			  VncViewerDepthType depth);
void vnc_viewer_close_connection(VncViewer *vnc);
void vnc_viewer_set_cursor(VncViewer *vnc, VncViewerCursorType cursor_type);

#endif

